@@ -109,6 +109,7 @@ static AtkAttributeSet* gail_notebook_page_get_run_attributes
 static AtkAttributeSet* gail_notebook_page_get_default_attributes
                                                    (AtkText           *text);
 static GtkWidget* get_label_from_notebook_page     (GailNotebookPage  *page);
+static GtkWidget* find_label_child (GtkContainer *container);
 
 static gpointer parent_class = NULL;
 
@@ -176,15 +177,36 @@ gail_notebook_page_class_init (GailNotebookPageClass *klass)
   gobject_class->finalize = gail_notebook_page_finalize;
 }
 
+static gint
+notify_child_added (gpointer data)
+{
+  GailNotebookPage *page;
+  AtkObject *atk_object, *atk_parent;
+
+  g_return_val_if_fail (GAIL_IS_NOTEBOOK_PAGE (data), FALSE);
+  page = GAIL_NOTEBOOK_PAGE (data);
+  atk_object = ATK_OBJECT (data);
+
+  /* The widget page->notebook may be deleted before this handler is called */
+  if (page->notebook == NULL)
+    return FALSE;
+  atk_parent = gtk_widget_get_accessible (GTK_WIDGET (page->notebook));
+  atk_object_set_parent (atk_object, atk_parent);
+  g_signal_emit_by_name (atk_parent, "children_changed::add", page->index, atk_object, NULL);
+  
+  return FALSE;
+}
+
 AtkObject*
 gail_notebook_page_new (GtkNotebook *notebook, 
                         gint        pagenum)
 {
   GObject *object;
-  AtkObject *atk_object, *atk_child;
+  AtkObject *atk_object;
   GailNotebookPage *page;
   GtkWidget *child;
   GtkWidget *label;
+  GList *list;
   
   g_return_val_if_fail (GTK_IS_NOTEBOOK (notebook), NULL);
 
@@ -200,15 +222,15 @@ gail_notebook_page_new (GtkNotebook *notebook,
   page->notebook = notebook;
   g_object_add_weak_pointer (G_OBJECT (page->notebook), (gpointer *)&page->notebook);
   page->index = pagenum;
+  list = g_list_nth (notebook->children, pagenum);
+  page->page = list->data;
   page->textutil = NULL;
   
   atk_object = ATK_OBJECT (page);
   atk_object->role = ATK_ROLE_PAGE_TAB;
   atk_object->layer = ATK_LAYER_WIDGET;
 
-  atk_child = gtk_widget_get_accessible (child);
-  atk_object_set_parent (atk_child, atk_object);
-  
+  g_idle_add (notify_child_added, atk_object);
   /*
    * We get notified of changes to the label
    */
@@ -792,6 +814,9 @@ get_label_from_notebook_page (GailNotebookPage *page)
   if (!notebook)
     return NULL;
 
+  if (!gtk_notebook_get_show_tabs (notebook))
+    return NULL;
+
   child = gtk_notebook_get_nth_page (notebook, page->index);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
 
@@ -801,24 +826,34 @@ get_label_from_notebook_page (GailNotebookPage *page)
     return child;
 
   if (GTK_IS_CONTAINER (child))
-    {
-      /*
-       * Child is not a label.	Search for a label in the container's children.
-       */
-      GList *children, *tmp_list;
+    child = find_label_child (GTK_CONTAINER (child));
 
-      children = gtk_container_get_children (GTK_CONTAINER (child));
+  return child;
+}
 
-      child = NULL;
-      for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next)
+static GtkWidget*
+find_label_child (GtkContainer *container)
+{
+  GList *children, *tmp_list;
+  GtkWidget *child;
+
+  children = gtk_container_get_children (container);
+
+  child = NULL;
+  for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next)
+    {
+      if (GTK_IS_LABEL (tmp_list->data))
+        {
+          child = GTK_WIDGET (tmp_list->data);
+          break;
+        }
+      else if (GTK_IS_CONTAINER (tmp_list->data))
         {
-          if (GTK_IS_LABEL (tmp_list->data))
-            {
-              child = GTK_WIDGET (tmp_list->data);
-              break;
-            }
+          child = find_label_child (GTK_CONTAINER (tmp_list->data));
+          if (child)
+            break;
         }
-     g_list_free (children); 
     }
+  g_list_free (children);
   return child;
 }
