@@ -49,7 +49,7 @@ struct _GailPanedClass
   GailContainerClass parent_class;
 };
 
-GtkAccessible *gail_paned_new(GtkWidget *widget);
+AtkObject* gail_paned_new (GtkWidget *widget);
 
 #ifdef __cplusplus
 }
