@@ -283,7 +283,7 @@ gail_range_finalize (GObject            *object)
       if (GAIL_ADJUSTMENT (range->adjustment)->adjustment)
         {
           g_signal_handlers_disconnect_by_func (GAIL_ADJUSTMENT (range->adjustment)->adjustment,
-                                                gail_range_value_changed,
+                                                (void *)gail_range_value_changed,
                                                 range);
         }
       g_object_unref (range->adjustment);
